@@ -838,6 +838,7 @@ S_gtk_container_child_get(USER_OBJECT_ s_object, USER_OBJECT_ s_child, USER_OBJE
 			SET_VECTOR_ELT(_result, i, VECTOR_ELT(
 				S_gtk_container_child_get_property(s_object, s_child, STRING_ELT(s_names, i)), 1));
 
+	UNPROTECT(1);
         return(_result);
 }
 
@@ -1209,8 +1210,11 @@ S_GtkTextBufferSerializeFunc(GtkTextBuffer* s_register_buffer, GtkTextBuffer* s_
 
   *s_length = GET_LENGTH(s_ans);
   
+  guint8 *ans = g_malloc_n(*s_length, sizeof(guint8));
+  memcpy(ans, RAW(s_ans), *s_length);
+
   UNPROTECT(1);
-  return(((guint8*)asCArray(s_ans, guint8, asCRaw)));
+  return ans;
 }
 
 /* need to return the x, y, and in_push params */
@@ -1251,7 +1255,7 @@ S_GtkBuilderConnectFuncDefault(GtkBuilder* builder, GObject* object,
   const gchar* signal_name, const gchar* handler_name, 
   GObject* connect_object, guint flags, gpointer s_user_data)
 {	
-  USER_OBJECT_ s_func = Rf_findFun(install(handler_name), R_GlobalEnv);
+  USER_OBJECT_ s_func = PROTECT(Rf_findFun(install(handler_name), R_GlobalEnv));
   GClosure *closure;
   
 	if (connect_object) /* FIXME: we can't use g_signal_connect_object */
@@ -1260,6 +1264,7 @@ S_GtkBuilderConnectFuncDefault(GtkBuilder* builder, GObject* object,
   closure = R_createGClosure(s_func, s_user_data);
 	((R_CallbackData *)closure->data)->userDataFirst = flags & G_CONNECT_SWAPPED;
   
+  UNPROTECT(1);
   g_signal_connect_closure(object, signal_name, closure, flags & G_CONNECT_AFTER);
 }
 
